@@ -1,9 +1,10 @@
 DEoptim.control <- function(VTR = -Inf, strategy = 2, bs = FALSE, NP = NA,
                             itermax = 200, CR = 0.5, F = 0.8, trace = TRUE,
                             initialpop = NULL, storepopfrom = itermax + 1,
-                            storepopfreq = 1, checkWinner = FALSE,
-                            avWinner = TRUE, p = 0.2, c = 0,
-                            reltol, steptol) {
+                            storepopfreq = 1,  p = 0.2, c = 0,
+                            reltol, steptol, parallelType = 0,
+                            packages = c(), parVar = c(), 
+                            foreachArgs = list() ) {
   if (itermax <= 0) {
     warning("'itermax' <= 0; set to default value 200\n", immediate. = TRUE)
     itermax <- 200
@@ -53,12 +54,14 @@ DEoptim.control <- function(VTR = -Inf, strategy = 2, bs = FALSE, NP = NA,
 
   list(VTR = VTR, strategy = strategy, NP = NP, itermax = itermax, CR
        = CR, F = F, bs = bs, trace = trace, initialpop = initialpop,
-       storepopfrom = storepopfrom, storepopfreq = storepopfreq,
-       checkWinner = checkWinner, avWinner = avWinner, p = p, c = c,
-       reltol = reltol, steptol = steptol)
+       storepopfrom = storepopfrom, storepopfreq = storepopfreq, p =
+       p, c = c, reltol = reltol, steptol = steptol, parallelType =
+       parallelType, packages = packages, parVar = parVar, foreachArgs =
+       foreachArgs)
 }
 
-DEoptim <- function(fn, lower, upper, control = DEoptim.control(), ...) {
+DEoptim <- function(fn, lower, upper, control = DEoptim.control(), ...,
+                    fnMap=NULL) {
   if (length(lower) != length(upper))
     stop("'lower' and 'upper' are not of same length")
   if (!is.vector(lower))
@@ -106,10 +109,84 @@ DEoptim <- function(fn, lower, upper, control = DEoptim.control(), ...) {
   ctrl$trace <- as.numeric(ctrl$trace)
   ctrl$specinitialpop <- as.numeric(ctrl$specinitialpop)
   ctrl$initialpop <- as.numeric(ctrl$initialpop)
+  if(ctrl$parallelType == 2) { ## use foreach 
+    use.foreach <- suppressMessages(require(foreach,quietly=TRUE))
+    if(!use.foreach)
+      stop("foreach package not available but parallelType set to 2")
+    if(!foreach:::getDoParRegistered()) {
+      foreach:::registerDoSEQ()
+    }
+    args <-  ctrl$foreachArgs
+    fnPop <- function(params, ...) {
+      my_chunksize <- ceiling(NROW(params)/foreach:::getDoParWorkers())
+      my_iter <- iter(params,by="row",chunksize=my_chunksize)
+      args$i <- my_iter
+      args$.combine <- c
+      args$.export <- "fn" 
+      do.call(foreach, args) %dopar% {
+        apply(i,1,fn,...)
+      }
+    }
+  }
+  else if(ctrl$parallelType == 1){ ## use parallel 
+    use.parallel <- suppressMessages(require(parallel,quietly=TRUE))
+    if(!use.parallel)
+      stop("parallel package not available but parallelType set to 1")
+    cl <- parallel:::makeCluster(parallel:::detectCores())
+    packFn <- function(packages) {
+      for(i in packages)
+        library(i, character.only = TRUE)
+    }
+    parallel:::clusterCall(cl, packFn, ctrl$packages)
+    parallel:::clusterExport(cl, ctrl$parVar)
+    fnPop <- function(params, ...) {
+      parallel:::parApply(cl=cl,params,1,fn,...)
+    }
+  }
+  else {  ## use regular for loop / apply
+    fnPop <- function(params, ...) {
+      apply(params,1,fn,...)
+    }
+  }
+
+  ## Mapping function
+  if(is.null(fnMap)) {
+    fnMapC <- function(params,...) params
+  } else {
+    fnMapC <- function(params,...) {
+      mappedPop <- t(apply(params,1,fnMap))   ## run mapping function
+      if(all(dim(mappedPop) != dim(params)))  ## check results
+        stop("mapping function did not return an object with ",
+             "dim NP x length(upper).")
+      dups <- duplicated(mappedPop)  ## check for duplicates
+      np <- NCOL(mappedPop)
+      tries <- 0
+      while(tries < 5 && any(dups)) {
+        ##print('dups!'); flush.console()
+        nd <- sum(dups)
+        ## generate new random population member
+        newPop <- matrix(runif(nd*np),ncol=np)
+        newPop <- rep(lower,each=nd) + newPop * rep(upper-lower,each=nd)
+        ## replace duplicate with _mapped_ random member
+        mappedPop[dups,] <- t(apply(newPop,1,fnMap))
+        dups <- duplicated(mappedPop)  ## re-check for duplicates
+        tries <- tries + 1
+      }
+      if(tries==5)
+        warning("Could not remove ",sum(dups)," duplicates from the mapped ",
+                "population in 5 tries. Evaluating population with duplicates.",
+                call.=FALSE, immediate.=TRUE)
+      ## memcpy fails if mappedPop isn't double (need TYPEOF switch in C?)
+      storage.mode(mappedPop) <- "double"
+      mappedPop
+    }
+  }
+
+  outC <- .Call("DEoptimC", lower, upper, fnPop, ctrl, new.env(), fnMapC, PACKAGE="DEoptim")
+
+  if(ctrl$parallelType == 1)
+    parallel:::stopCluster(cl) 
   
-  outC <- .Call("DEoptimC", lower, upper, fn, ctrl, new.env(),
-               PACKAGE = "DEoptim")
-  ##
   if (length(outC$storepop) > 0) {
     nstorepop <- floor((outC$iter - ctrl$storepopfrom) / ctrl$storepopfreq)
     storepop <- list()
@@ -126,39 +203,29 @@ DEoptim <- function(fn, lower, upper, control = DEoptim.control(), ...) {
     storepop = NULL
   }
 
-  ## optim
-#  bestmem <- as.numeric(outC$bestmem)
-#  names(bestmem) <- nam
   names(outC$bestmem) <- nam
-#  bestval <- as.numeric(outC$bestval)
-#  nfeval <- as.numeric(outC$nfeval)
   iter <- max(1,as.numeric(outC$iter))
 
-  ## member
   names(lower) <- names(upper) <- nam
   bestmemit <- matrix(outC$bestmemit[1:(iter * ctrl$npar)],
                       nrow = iter, ncol = ctrl$npar, byrow = TRUE)
 
   dimnames(bestmemit) <- list(1:iter, nam)
-#  dimnames(outC$bestmemit) <- list(1:outC$iter, nam)
-#  bestvalit <- as.numeric(outC$bestvalit[1:iter])
-#  pop <- matrix(outC$pop, nrow = ctrl$NP, ncol = ctrl$npar,
-#                byrow = TRUE)
   storepop <- as.list(storepop)
 
   outR <- list(optim = list(
-              bestmem = outC$bestmem,
-              bestval = outC$bestval,
-              nfeval = outC$nfeval,
-              iter = outC$iter),
-            member = list(
-              lower = lower,
-              upper = upper,
-              bestmemit = bestmemit,
-              bestvalit = outC$bestvalit,
-              pop = t(outC$pop),
-              storepop = storepop))
-
+                 bestmem = outC$bestmem,
+                 bestval = outC$bestval,
+                 nfeval = outC$nfeval,
+                 iter = outC$iter),
+               member = list(
+                 lower = lower,
+                 upper = upper,
+                 bestmemit = bestmemit,
+                 bestvalit = outC$bestvalit,
+                 pop = t(outC$pop),
+                 storepop = storepop))
+  
   attr(outR, "class") <- "DEoptim"
   return(outR)
 }
