@@ -41,7 +41,7 @@ void devol(double VTR, double d_weight, double fcross, int i_bs_flag,
            double d_reltol, int i_steptol, SEXP fnMap);
 void permute(int ia_urn2[], int i_urn2_depth, int i_NP, int i_avoid, int ia_urn1[]);
 double evaluate(long *l_nfeval, SEXP par, SEXP fcall, SEXP env);
-SEXP popEvaluate(long *l_nfeval, SEXP parMat, SEXP fcall, SEXP env);
+SEXP popEvaluate(long *l_nfeval, SEXP parMat, SEXP fcall, SEXP env, int incrementEval);
 
 
 /*------General functions-----------------------------------------*/
@@ -174,11 +174,9 @@ void devol(double VTR, double d_weight, double d_cross, int i_bs_flag,
   PROTECT(sexp_gta_popP = allocMatrix(REALSXP, i_NP, i_D)); P++; /* FIXME THIS HAD 2x the rows!!! */
   PROTECT(sexp_gta_oldP = allocMatrix(REALSXP, i_NP, i_D)); P++;
   PROTECT(sexp_gta_newP = allocMatrix(REALSXP, i_NP, i_D)); P++;
-  PROTECT(sexp_map_pop  = allocMatrix(REALSXP, i_NP, i_D)); P++;
   double *ngta_popP = REAL(sexp_gta_popP); /* FIXME THIS HAD 2x the rows!!! */
   double *ngta_oldP = REAL(sexp_gta_oldP);
   double *ngta_newP = REAL(sexp_gta_newP);
-  double *nmap_pop  = REAL(sexp_map_pop);
 
   /* Data structures for objective function values associated with
    * parameter vectors */
@@ -262,10 +260,10 @@ void devol(double VTR, double d_weight, double d_cross, int i_bs_flag,
         ngta_popP[i+i_NP*j] = initialpop[i][j];
     }
   }
-  PROTECT(sexp_map_pop  = popEvaluate(l_nfeval, sexp_gta_popP, fnMap, rho));
-  memcpy(REAL(sexp_gta_popP), REAL(sexp_map_pop), i_NP * i_D * sizeof(double));
+  PROTECT(sexp_map_pop  = popEvaluate(l_nfeval, sexp_gta_popP, fnMap, rho, 0));
+  memmove(REAL(sexp_gta_popP), REAL(sexp_map_pop), i_NP * i_D * sizeof(double)); // valgrind reports memory overlap here
   UNPROTECT(1);  // sexp_map_pop
-  PROTECT(sexp_gta_popC = popEvaluate(l_nfeval, sexp_gta_popP,  fcall, rho));
+  PROTECT(sexp_gta_popC = popEvaluate(l_nfeval, sexp_gta_popP,  fcall, rho, 1));
   ngta_popC = REAL(sexp_gta_popC);
   for (i = 0; i < i_NP; i++) {
     if (i == 0 || ngta_popC[i] <= t_bestC) {
@@ -423,10 +421,10 @@ void devol(double VTR, double d_weight, double d_cross, int i_bs_flag,
     /*------Trial mutation now in t_tmpP-----------------*/
     /* evaluate mutated population */
     if(i_iter > 1) UNPROTECT(1);  // previous iteration's sexp_t_tmpC
-    PROTECT(sexp_map_pop = popEvaluate(l_nfeval, sexp_t_tmpP,  fnMap, rho));
-    memcpy(REAL(sexp_t_tmpP), REAL(sexp_map_pop), i_NP * i_D * sizeof(double));
+    PROTECT(sexp_map_pop = popEvaluate(l_nfeval, sexp_t_tmpP,  fnMap, rho, 0));
+    memmove(REAL(sexp_t_tmpP), REAL(sexp_map_pop), i_NP * i_D * sizeof(double)); // valgrind reports memory overlap here
     UNPROTECT(1);  // sexp_map_pop
-    PROTECT(sexp_t_tmpC  = popEvaluate(l_nfeval, sexp_t_tmpP, fcall, rho));
+    PROTECT(sexp_t_tmpC  = popEvaluate(l_nfeval, sexp_t_tmpP, fcall, rho, 1));
     nt_tmpC = REAL(sexp_t_tmpC);
 
     /* compare old pop with mutated pop */
